program rdqtese, eclass
	/* This program is the front-office for RD quantile treatment effects. It is the one called by the user, 
	it calls the back-office programs to compute bandwidths and estimates, and reports results */
	/* rdqteu is the same as rdqte except it uses the uniform kernel */
	/* version se calculates analytical standard errors (still using uniform kernel) */
	version 10
	syntax varlist(min=2 max=2 numeric) [if] [in]  , RUNNING(varname) QUANTILES(numlist) [BSREPS(int -1)] [LEVEL(int 90)] [BWs(numlist)] 
	
	/*
	 syntax: rdqtep2 depvar treatvar, running(runningvar) quantiles(indexlist) [bsreps(# of bootstrap iterations) level(cilevel) bws(list of mean bandwidths)]
	example: rdqtep y d, running(r) quantiles(.1(.1).9) bsreps(500) level(90) bws(.2 .2 .2 .2 .2 .2)
	Comments:
	the treatment variable must be binary
	the running variable must be normalized so the RD threshold is at zero, and the probability of treatment must increase discontinuously for
	the running variable *eaual* or greater than zero
	bws should be a list of six bandwidths for estimating the following conditional means:
		* 1. hd_1= for E[D|r=0] from above
		* 2. h_11= for E[Y|r=0,D=1] from above
		* 3. h_10= for E[Y|r=0,D=0] from above
		* 4. hd_0= for E[D|r=0] from below
		* 5. h_01= for E[Y|r=0,D=1] from below
		* 6. h_00= for E[Y|r=0,D=0] from below
	A negative bandwidth indicates a design with zero observations in the relevant cell
	A bandwidth of 999 indicates no variance in treatment status in the cell. 
	*/
	*to make sure moremata is installed:
	capture mata: mm_matlist(2)
	if _rc!=0 display in gr `""moremata" package not installed. Try "net search moremata"."'
	else {
	marksample touse
	preserve
	keep if `touse'
	tempname ci_bc ci_percentile ci_normal lqtec lqte Q q qlength conv0 conv1 h0 hy1 hy0 se
	tempvar z
	local `qlength'=0
	foreach quant of numlist `quantiles' {
		local ++`qlength'
	}
	*Initialize quantile matrix
	matrix `Q'=J(``qlength'',1,0)	
	
	*Populate quantile matrix
	local `q'=0
	foreach quant of numlist `quantiles' {
		local ++`q'
		matrix `Q'[``q'',1]=`quant'
	}
	
	
	tokenize `varlist'
	local y "`1'"
	local d "`2'"
	local r "`running'"
	gen `z'=`r'>=0
	* Compute optimal bandwidths if not provided:
	if "`bws'"=="" {	
		/* amount of data to keep in block quartic regression: */ 
		local delta=.95			
		qui findbw `y' `d',running( `r') delta( `delta')
		local hmeans `r(hmeans)'
		scalar `hy1'=r(hy1)
		scalar `hy0'=r(hy0)
	}
	else{ 
		local hmeans `bws' 
		scalar `hy1'=0
		scalar `hy0'=0
	}
	display in gr "Mean bandwidths to be used: `hmeans'"
 
	if `bsreps'!=-1 {
		capture  qui bootstrap, reps(`bsreps') level(`level'): _rdqtese `varlist' ///
			,running(`running') quantiles(`quantiles') hmeans("`hmeans'")
		matrix `lqtec'=e(b)
		matrix `lqte'=`lqtec''
		matrix `ci_bc'=e(ci_bc)'
		matrix `ci_percentile'=e(ci_percentile)'
		matrix `ci_normal'=e(ci_normal)'
		matrix results=`Q',`ci_percentile'[1...,1],`lqte',`ci_percentile'[1...,2]
		matrix colnames results=quantile lb lqte ub
	}
	else {
		/* qui */  _rdqtese `varlist' ,running(`running') quantiles(`quantiles') hmeans("`hmeans'")
		matrix `lqtec'=e(b)
		matrix `lqte'=`lqtec''
		matrix `se'=e(se)
		matrix results=`Q',`lqte',`se'
		matrix colnames results=quantile lqte se
		
	}
	
	*matrix rownames results=`quantiles'
	scalar `conv0'=e(conv0)
	scalar `conv1'=e(conv1)
	ereturn post,esample(`touse')
	ereturn matrix lqte=`lqte'
	ereturn matrix quantiles=`Q'
	ereturn matrix se=`se'	
	if `bsreps'!=-1 {
		ereturn matrix ci_bc=`ci_bc'
		ereturn matrix ci_percentile=`ci_percentile'
		ereturn matrix ci_normal=`ci_normal'
	}	
	
	ereturn local cmd "rdqteu" 
	ereturn scalar conv0 =`conv0'
	ereturn scalar conv1 =`conv1'
	ereturn scalar hy1=`hy1'
	ereturn scalar hy0=`hy0'
	ereturn local hmeans "`hmeans'"
	display in gr _n _n "RD Local Quantile Treatment Effects estimation results"
	mat li results
	if (`conv0'!=0 | `conv1'!=0) {
		display in ye "distribution function inversion did not converge"
	}
	else {
		display in ye "distribution function inversion converged successfully"
	} 
	matrix drop results
	restore
	}	
end

program findbw, rclass
	* Back-office program using plug in method to find bandwidths
	version 10
	syntax varlist(min=2 max=2 numeric) [if] [in]  , RUNNING(varname) [delta(real .5)]  
	marksample touse
	tempname h0 zval del rbw fr fr1 fr0 fr11 fr10 fr01 fr00 ncurr bk ak ///
	prz1 prz0 prz11 prz10 prz01 prz00 a0 a1 a2
	tempvar z kr kLr
	
	scalar `del'=`delta'
	* Find bandwidths for means by cross validation.
	tokenize `varlist'
	local y "`1'"
	local d "`2'"
	local r "`running'"
	gen `z'=`r'>=0
	/* We have 6 different sets of bandwidths to generate. The hmeans macro has the following
	six bandwidths for means, in order*/
		* 1. hd_1= for E[D|r=0] from above
		* 2. h_11= for E[Y|r=0,D=1] from above
		* 3. h_10= for E[Y|r=0,D=0] from above
		* 4. hd_0= for E[D|r=0] from below
		* 5. h_01= for E[Y|r=0,D=1] from below
		* 6. h_00= for E[Y|r=0,D=0] from below
	
	*define constants for uniform kernel:
	scalar `bk'=4
	scalar `ak'=1/6
	
	*Step one: estimate f[R|D=d,Z=z](0):
	* first estimate unconditional density using simple kernel density estimation
	gen `kr'=.
	gen `kLr'=.
	sum `r'
	scalar `rbw'=r(N)^(-1/5)*r(sd)*1.06
	replace `kr'=1/`rbw'*normalden(`r'/`rbw')
	sum `kr'
	scalar `fr'=r(mean)
	
	*Now use baye's rule to condition the above density on Z=zval
	sum `z'
	scalar `prz1'=r(mean)
	scalar `prz0'=1-r(mean)
	scalar `fr1'=`fr'/`prz1'
	scalar `fr0'=`fr'/`prz0'
	
	*Now use a boundary kernel to estimate density conditional on Z==Zval & D==Dval
	*define constants for use in the boundary kernel:
	scalar `a0'=.5
	scalar `a1'=-1/sqrt(2*3.14159265359)
	scalar `a2'=.5	
	foreach Zval in 1 0 {
		foreach Dval in 1 0 {
			sum `r' if `z'==`Zval' & `d'==`Dval'
			if r(N)>0 {
				scalar `rbw'=r(N)^(-1/5)*r(sd)*1.06
				replace `kr'=1/`rbw'*normalden(`r'/`rbw')
				replace `kLr'=(`a2'-`a1'*`r')*`kr'/(`a0'*`a2'-`a1'^2)
				sum `kLr'
				scalar `fr`Zval'`Dval''=r(mean)
			}
			else scalar `fr`Zval'`Dval''=-1
		}
	}
	
		
	foreach Zval in 1 0 {
		sum `r' if `z'==`Zval'
		scalar `ncurr'=r(N)
		getm2sig2 `d' `r' if `z'==`Zval', zval( `Zval') delta( `delta')
		if r(sig20)!=0 local hcurr=`ncurr'^(-1/5)*(`bk'*r(sig20)/`fr`Zval''/`ak'^2/r(m20)^2)^(1/5)
		else local hcurr=999
		local hmac `hmac' `hcurr'
		display "hmac has:" "`hmac'"

		getm2sig2 `y' `r' if `z'==`Zval', zval( `Zval') delta( `delta')
		local hcurr=`ncurr'^(-1/5)*(`bk'*r(sig20)/`fr`Zval''/`ak'^2/r(m20)^2)^(1/5)
		return scalar hy`Zval'=`hcurr'
		foreach Dval in 1 0 {
			sum `r' if `z'==`Zval' & `d'==`Dval'
			if r(N)>0 {
				scalar `ncurr'=r(N)
				getm2sig2 `y' `r' if `z'==`Zval' & `d'==`Dval', zval( `Zval') delta( `delta')
				local hcurr=`ncurr'^(-1/5)*(`bk'*r(sig20)/`fr`Zval'`Dval''/`ak'^2/r(m20)^2)^(1/5)
				local hmac `hmac' `hcurr'
				display "hmac has:" "`hmac'"
			}
			else {
				local hcurr=-1
				local hmac `hmac' `hcurr'
				display "hmac has:" "`hmac'"
			}
		}	
	}

	return local hmeans "`hmac'"
	
end

program getm2sig2, rclass
	/* estimate m''(0) and sig2(0) for use in plug in method
	*/
	version 10
	syntax varlist(min=2 max=2 numeric) [if] [in]  , ZVAL(int) [delta(real .5)]  
	marksample touse
	tempname rmax m20 sig20
	tempvar rsc rsc2 rsc3 rsc4 w absr
	tokenize `varlist'
	local y "`1'"
	local r "`2'"
	preserve
	keep if `touse'
	if `zval'==1 {
		keep if `r'>=0
		gsort + `r'
	}
	else {
		keep if `r'<0
		gsort - `r'
	}
	local N=_N
	local xd=`r'[`N'*`delta']
	if `zval'==1 {
		count if `r'<=`xd'
		local Nd=r(N)
	}
	else {
		count if `r'>=`xd'
		local Nd=r(N)
	}
	* generate weight for r within the block:
	gen `w'=(abs(`r')<=abs(`xd'))
	* scale running variable so polynomials don't get out of control:
	gen `absr'=abs(`r')
	sum `absr'
	scalar `rmax'=r(max)
	
	gen `rsc'=`r'/`rmax'
	gen `rsc2'=`rsc'^2
	gen `rsc3'=`rsc'^3
	gen `rsc4'=`rsc'^4
	/* First step: estimate m''(0) and sig2(0) by local quartic regression */
	reg `y' `rsc' `rsc2' `rsc3' `rsc4' [aw=`w']
	scalar `m20'=2/(`rmax'^2)*_b[`rsc2']
	scalar `sig20'=e(rmse)^2
	display "m20 is " `m20'
	display "sig20 is " `sig20'
		
	return scalar m20=`m20'
	return scalar sig20=`sig20'
	restore
end
